<?php

namespace Drupal\terraform\Service;

use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Http\ClientFactory;
use Drupal\Core\Lock\LockBackendInterface;

/**
 * TerraformServiceMock service interacts with the Terraform API.
 */
class TerraformServiceMock extends TerraformService {

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructs a new TerraformService object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   An entity type manager instance.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   A configuration factory.
   * @param \Drupal\Core\Http\ClientFactory $client_factory
   *   The http client.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager.
   * @param \Drupal\Core\Lock\LockBackendInterface $lock
   *   The lock interface.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    ClientFactory $client_factory,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    LockBackendInterface $lock
  ) {
    parent::__construct(
      $entity_type_manager,
      $config_factory,
      $client_factory,
      $cloud_config_plugin_manager,
      $lock
    );

    $this->config = $config_factory->get('terraform.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function describeWorkspaces(array $params = []) {
    return json_decode($this->config->get('terraform_mock_data'), TRUE)['describeWorkspaces'] ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function createWorkspace(array $params) {
    return json_decode($this->config->get('terraform_mock_data'), TRUE)['createWorkspace'] ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteWorkspace($name) {
    return $this->getMockData(__METHOD__);
  }

  /**
   * {@inheritdoc}
   */
  public function describeRuns(array $params = []) {
    return $this->getMockData(__METHOD__);
  }

  /**
   * {@inheritdoc}
   */
  public function describeStates(array $params = []) {
    return $this->getMockData(__METHOD__);
  }

  /**
   * Get mock data for a method.
   *
   * @param string $method_name
   *   The method name.
   *
   * @return array
   *   An array of the mock data for a method.
   */
  private function getMockData($method_name) {
    return json_decode($this->config->get('terraform_mock_data'), TRUE)[$method_name] ?? [];
  }

}
